=== modified file 'client/drizzle.cc'
--- client/drizzle.cc	2011-11-24 00:05:44 +0000
+++ client/drizzle.cc	2011-11-26 23:14:59 +0000
@@ -282,7 +282,7 @@
 typedef enum enum_info_type INFO_TYPE;
 
 static drizzle_st *drizzle= NULL;      /* The library handle */
-static drizzle_con_st con;      /* The connection */
+static drizzle_con_st *con= NULL;      /* The connection */
 static bool ignore_errors= false, quick= false,
   connected= false, opt_raw_data= false, unbuffered= false,
   output_tables= false, opt_rehash= true, skip_updates= false,
@@ -1188,29 +1188,29 @@
 static bool server_shutdown(void)
 {
   drizzle_result_st result;
-  drizzle_return_t ret;
 
   if (verbose)
   {
     printf(_("shutting down drizzled"));
     if (opt_drizzle_port > 0)
+    {
       printf(_(" on port %d"), opt_drizzle_port);
+    }
     printf("... ");
   }
 
-  if (drizzle_shutdown(&con, &result, DRIZZLE_SHUTDOWN_DEFAULT,
-                       &ret) == NULL || ret != DRIZZLE_RETURN_OK)
+  drizzle_return_t ret;
+  if (drizzle_shutdown(con, &result, DRIZZLE_SHUTDOWN_DEFAULT, &ret) == NULL or
+      ret != DRIZZLE_RETURN_OK)
   {
     if (ret == DRIZZLE_RETURN_ERROR_CODE)
     {
-      fprintf(stderr, _("shutdown failed; error: '%s'"),
-              drizzle_result_error(&result));
+      fprintf(stderr, _("shutdown failed; error: '%s'"), drizzle_result_error(&result));
       drizzle_result_free(&result);
     }
     else
     {
-      fprintf(stderr, _("shutdown failed; error: '%s'"),
-              drizzle_con_error(&con));
+      fprintf(stderr, _("shutdown failed; error: '%s'"), drizzle_con_error(con));
     }
     return false;
   }
@@ -1218,7 +1218,9 @@
   drizzle_result_free(&result);
 
   if (verbose)
+  {
     printf(_("done\n"));
+  }
 
   return true;
 }
@@ -1234,7 +1236,7 @@
     printf("... ");
   }
 
-  if (drizzle_kill(&con, &result, query_id,
+  if (drizzle_kill(con, &result, query_id,
                    &ret) == NULL || ret != DRIZZLE_RETURN_OK)
   {
     if (ret == DRIZZLE_RETURN_ERROR_CODE)
@@ -1245,8 +1247,7 @@
     }
     else
     {
-      fprintf(stderr, _("kill failed; error: '%s'"),
-              drizzle_con_error(&con));
+      fprintf(stderr, _("kill failed; error: '%s'"), drizzle_con_error(con));
     }
     return false;
   }
@@ -1272,7 +1273,7 @@
   drizzle_result_st result;
   drizzle_return_t ret;
 
-  if (drizzle_ping(&con, &result, &ret) != NULL && ret == DRIZZLE_RETURN_OK)
+  if (drizzle_ping(con, &result, &ret) != NULL && ret == DRIZZLE_RETURN_OK)
   {
     if (opt_silent < 2)
       printf(_("drizzled is alive\n"));
@@ -1287,8 +1288,7 @@
     }
     else
     {
-      fprintf(stderr, _("drizzled won't answer to ping, error: '%s'"),
-              drizzle_con_error(&con));
+      fprintf(stderr, _("drizzled won't answer to ping, error: '%s'"), drizzle_con_error(con));
     }
     return false;
   }
@@ -1840,10 +1840,10 @@
   glob_buffer->reserve(512);
 
   snprintf(&output_buff[0], output_buff.size(),
-          _("Your Drizzle connection id is %u\nConnection protocol: %s\nServer version: %s\n"),
-          drizzle_con_thread_id(&con),
-          opt_protocol.c_str(),
-          server_version_string(&con));
+           _("Your Drizzle connection id is %u\nConnection protocol: %s\nServer version: %s\n"),
+           drizzle_con_thread_id(con),
+           opt_protocol.c_str(),
+           server_version_string(con));
   put_info(&output_buff[0], INFO_INFO, 0, 0);
 
 
@@ -1898,7 +1898,7 @@
 
 void drizzle_end(int sig)
 {
-  drizzle_con_free(&con);
+  drizzle_con_free(con);
   drizzle_free(drizzle);
   if (!status.getBatch() && !quick && histfile)
   {
@@ -1937,39 +1937,38 @@
 extern "C"
 void handle_sigint(int sig)
 {
-  char kill_buffer[40];
-  boost::scoped_ptr<drizzle_con_st> kill_drizzle(new drizzle_con_st);
-  drizzle_result_st res;
-  drizzle_return_t ret;
-
   /* terminate if no query being executed, or we already tried interrupting */
-  if (!executing_query || interrupted_query)
-  {
-    goto err;
-  }
-
-  if (drizzle and drizzle_con_add_tcp(drizzle, kill_drizzle.get(), current_host.c_str(),
-    opt_drizzle_port, current_user.c_str(), opt_password.c_str(), NULL,
-    use_drizzle_protocol ? DRIZZLE_CON_EXPERIMENTAL : DRIZZLE_CON_MYSQL) == NULL)
-  {
-    goto err;
-  }
-
-  /* kill_buffer is always big enough because max length of %lu is 15 */
-  sprintf(kill_buffer, "KILL /*!50000 QUERY */ %u",
-          drizzle_con_thread_id(&con));
-
-  if (drizzle_query_str(kill_drizzle.get(), &res, kill_buffer, &ret) != NULL)
-    drizzle_result_free(&res);
-
-  drizzle_con_free(kill_drizzle.get());
-  tee_fprintf(stdout, _("Query aborted by Ctrl+C\n"));
-
-  interrupted_query= 1;
-
-  return;
-
-err:
+  if (executing_query == false or interrupted_query)
+  { }
+  else if (drizzle)
+  {
+    drizzle_con_st *kill_drizzle_con;
+    if ((kill_drizzle_con= drizzle_con_add_tcp(drizzle,
+                                               current_host.c_str(), opt_drizzle_port,
+                                               current_user.c_str(), opt_password.c_str(), NULL,
+                                               use_drizzle_protocol ? DRIZZLE_CON_EXPERIMENTAL : DRIZZLE_CON_MYSQL)))
+    {
+      /* kill_buffer is always big enough because max length of %lu is 15 */
+      char kill_buffer[40];
+      snprintf(kill_buffer, sizeof(kill_buffer), "KILL /*!50000 QUERY */ %u", drizzle_con_thread_id(kill_drizzle_con));
+
+      drizzle_return_t ret;
+      drizzle_result_st res;
+      if ((drizzle_query_str(kill_drizzle_con, &res, kill_buffer, &ret)))
+      {
+        drizzle_result_free(&res);
+      }
+
+      drizzle_con_free(kill_drizzle_con);
+
+      tee_fprintf(stdout, _("Query aborted by Ctrl+C\n"));
+
+      interrupted_query= true;
+
+      return;
+    }
+  }
+
   drizzle_end(sig);
 }
 
@@ -2206,7 +2205,7 @@
       }
     }
     if (!*ml_comment && inchar == '\\' &&
-        !(*in_string && (drizzle_con_status(&con) & DRIZZLE_CON_STATUS_NO_BACKSLASH_ESCAPES)))
+        !(*in_string && (drizzle_con_status(con) & DRIZZLE_CON_STATUS_NO_BACKSLASH_ESCAPES)))
     {
       // Found possbile one character command like \c
 
@@ -2622,7 +2621,7 @@
   /* hash Drizzle functions (to be implemented) */
 
   /* hash all database names */
-  if (drizzle_query_str(&con, &databases, "select schema_name from information_schema.schemata", &ret) != NULL)
+  if (drizzle_query_str(con, &databases, "select schema_name from information_schema.schemata", &ret) != NULL)
   {
     if (ret == DRIZZLE_RETURN_OK)
     {
@@ -2648,8 +2647,7 @@
   query.append(current_db);
   query.append("' order by table_name");
   
-  if (drizzle_query(&con, &fields, query.c_str(), query.length(),
-                    &ret) != NULL)
+  if (drizzle_query(con, &fields, query.c_str(), query.length(), &ret) != NULL)
   {
     if (ret == DRIZZLE_RETURN_OK &&
         drizzle_result_buffer(&fields) == DRIZZLE_RETURN_OK)
@@ -2714,7 +2712,7 @@
   current_db.erase();
   current_db= "";
   /* In case of error below current_db will be NULL */
-  if (drizzle_query_str(&con, &res, "SELECT DATABASE()", &ret) != NULL)
+  if (drizzle_query_str(con, &res, "SELECT DATABASE()", &ret) != NULL)
   {
     if (ret == DRIZZLE_RETURN_OK &&
         drizzle_result_buffer(&res) == DRIZZLE_RETURN_OK)
@@ -2740,12 +2738,12 @@
   for (uint32_t retry=0;; retry++)
   {
     int error;
-    if (drizzle_query(&con,result,buf,length,&ret) != NULL &&
+    if (drizzle_query(con, result, buf, length, &ret) != NULL and
         ret == DRIZZLE_RETURN_OK)
     {
       return 0;
     }
-    error= put_error(&con, result);
+    error= put_error(con, result);
 
     if (ret == DRIZZLE_RETURN_ERROR_CODE)
     {
@@ -2767,14 +2765,17 @@
 int drizzleclient_store_result_for_lazy(drizzle_result_st *result)
 {
   if (drizzle_result_buffer(result) == DRIZZLE_RETURN_OK)
+  {
     return 0;
+  }
 
-  if (drizzle_con_error(&con)[0])
+  if (drizzle_con_error(con)[0])
   {
-    int ret= put_error(&con, result);
+    int ret= put_error(con, result);
     drizzle_result_free(result);
     return ret;
   }
+
   return 0;
 }
 
@@ -2838,7 +2839,7 @@
   uint32_t      error_code= 0;
   int           err= 0;
 
-  interrupted_query= 0;
+  interrupted_query= false;
 
   /* Remove garbage for nicer messages */
   remove_cntrl(buffer);
@@ -2868,7 +2869,7 @@
   }
 
   timer=start_timer();
-  executing_query= 1;
+  executing_query= true;
   error= drizzleclient_real_query_for_lazy(buffer->c_str(),buffer->length(),&result, &error_code);
 
   if (status.getAddToHistory())
@@ -2891,7 +2892,7 @@
     {
       if (drizzle_column_buffer(&result) != DRIZZLE_RETURN_OK)
       {
-        error= put_error(&con, &result);
+        error= put_error(con, &result);
         goto end;
       }
     }
@@ -2930,7 +2931,7 @@
                 (long) drizzle_result_row_count(&result));
         end_pager();
         if (drizzle_result_error_code(&result))
-          error= put_error(&con, &result);
+          error= put_error(con, &result);
       }
     }
     else if (drizzle_result_affected_rows(&result) == ~(uint64_t) 0)
@@ -2965,9 +2966,9 @@
       fflush(stdout);
     drizzle_result_free(&result);
 
-    if (drizzle_con_status(&con) & DRIZZLE_CON_STATUS_MORE_RESULTS_EXISTS)
+    if (drizzle_con_status(con) & DRIZZLE_CON_STATUS_MORE_RESULTS_EXISTS)
     {
-      if (drizzle_result_read(&con, &result, &ret) == NULL ||
+      if (drizzle_result_read(con, &result, &ret) == NULL ||
           ret != DRIZZLE_RETURN_OK)
       {
         if (ret == DRIZZLE_RETURN_ERROR_CODE)
@@ -2976,14 +2977,16 @@
           drizzle_result_free(&result);
         }
 
-        error= put_error(&con, NULL);
+        error= put_error(con, NULL);
         goto end;
       }
     }
 
-  } while (drizzle_con_status(&con) & DRIZZLE_CON_STATUS_MORE_RESULTS_EXISTS);
+  } while (drizzle_con_status(con) & DRIZZLE_CON_STATUS_MORE_RESULTS_EXISTS);
   if (err >= 1)
-    error= put_error(&con, NULL);
+  {
+    error= put_error(con, NULL);
+  }
 
 end:
 
@@ -2991,20 +2994,20 @@
   if (show_warnings == 1 && (warnings >= 1 || error))
     print_warnings(error_code);
 
-  if (!error && !status.getBatch() &&
-      drizzle_con_status(&con) & DRIZZLE_CON_STATUS_DB_DROPPED)
+  if (!error && !status.getBatch() and
+      drizzle_con_status(con) & DRIZZLE_CON_STATUS_DB_DROPPED)
   {
     get_current_db();
   }
 
-  executing_query= 0;
+  executing_query= false;
   return error;        /* New command follows */
 }
 
 
 static void init_pager()
 {
-  if (!opt_nopager)
+  if (opt_nopager == false)
   {
     if (!(PAGER= popen(pager.c_str(), "w")))
     {
@@ -3013,7 +3016,9 @@
     }
   }
   else
+  {
     PAGER= stdout;
+  }
 }
 
 static void end_pager()
@@ -3027,8 +3032,11 @@
 {
   FILE* new_outfile;
   if (opt_outfile)
+  {
     end_tee();
-  if (!(new_outfile= fopen(file_name, "a")))
+  }
+
+  if ((new_outfile= fopen(file_name, "a")) == NULL)
   {
     tee_fprintf(stdout, _("Error logging to file '%s'\n"), file_name);
     return;
@@ -3261,7 +3269,7 @@
       cur= drizzle_row_buffer(result, &ret);
       if (ret != DRIZZLE_RETURN_OK)
       {
-        (void)put_error(&con, result);
+        (void)put_error(con, result);
         break;
       }
     }
@@ -3467,7 +3475,7 @@
       cur= drizzle_row_buffer(result, &ret);
       if (ret != DRIZZLE_RETURN_OK)
       {
-        (void)put_error(&con, result);
+        (void)put_error(con, result);
         break;
       }
     }
@@ -3633,7 +3641,7 @@
       cur= drizzle_row_buffer(result, &ret);
       if (ret != DRIZZLE_RETURN_OK)
       {
-        (void)put_error(&con, result);
+        (void)put_error(con, result);
         break;
       }
     }
@@ -3877,7 +3885,7 @@
 
   if (connected)
   {
-    sprintf(buff, _("Connection id:    %u"), drizzle_con_thread_id(&con));
+    sprintf(buff, _("Connection id:    %u"), drizzle_con_thread_id(con));
     put_info(buff,INFO_INFO,0,0);
     sprintf(buff, _("Current schema: %.128s\n"),
             !current_db.empty() ? current_db.c_str() : _("*** NONE ***"));
@@ -4028,21 +4036,25 @@
       return opt_reconnect ? -1 : 1;                        // Fatal error
     for (bool try_again= true; try_again; try_again= false)
     {
-      if (drizzle_select_db(&con,&result,tmp,&ret) == NULL ||
+      if (drizzle_select_db(con, &result, tmp, &ret) == NULL ||
           ret != DRIZZLE_RETURN_OK)
       {
         if (ret == DRIZZLE_RETURN_ERROR_CODE)
         {
-          int error= put_error(&con, &result);
+          int error= put_error(con, &result);
           drizzle_result_free(&result);
           return error;
         }
 
         if (ret != DRIZZLE_RETURN_LOST_CONNECTION || !try_again)
-          return put_error(&con, NULL);
+        {
+          return put_error(con, NULL);
+        }
 
         if (reconnect())
+        {
           return opt_reconnect ? -1 : 1;                      // Fatal error
+        }
       }
       else
         drizzle_result_free(&result);
@@ -4069,19 +4081,17 @@
     printf("... ");
   }
 
-  if (drizzle_shutdown(&con, &result, DRIZZLE_SHUTDOWN_DEFAULT,
+  if (drizzle_shutdown(con, &result, DRIZZLE_SHUTDOWN_DEFAULT,
                        &ret) == NULL || ret != DRIZZLE_RETURN_OK)
   {
     if (ret == DRIZZLE_RETURN_ERROR_CODE)
     {
-      fprintf(stderr, _("shutdown failed; error: '%s'"),
-              drizzle_result_error(&result));
+      fprintf(stderr, _("shutdown failed; error: '%s'"), drizzle_result_error(&result));
       drizzle_result_free(&result);
     }
     else
     {
-      fprintf(stderr, _("shutdown failed; error: '%s'"),
-              drizzle_con_error(&con));
+      fprintf(stderr, _("shutdown failed; error: '%s'"), drizzle_con_error(con));
     }
     return false;
   }
@@ -4179,7 +4189,7 @@
   if (connected)
   {
     connected= 0;
-    drizzle_con_free(&con);
+    drizzle_con_free(con);
     drizzle_free(drizzle);
   }
 
@@ -4189,22 +4199,22 @@
     return 1;
   }
 
-  if (drizzle_con_add_tcp(drizzle, &con, (char *)host.c_str(),
-                          opt_drizzle_port, (char *)user.c_str(),
-                          (char *)password.c_str(), (char *)database.c_str(),
-                          global_con_options) == NULL)
+  if ((con= drizzle_con_add_tcp(drizzle, host.c_str(),
+                                opt_drizzle_port, user.c_str(),
+                                password.c_str(), database.c_str(),
+                                global_con_options)) == NULL)
   {
-    (void) put_error(&con, NULL);
+    (void) put_error(con, NULL);
     (void) fflush(stdout);
     return 1;
   }
 
-  if ((ret= drizzle_con_connect(&con)) != DRIZZLE_RETURN_OK)
+  if ((ret= drizzle_con_connect(con)) != DRIZZLE_RETURN_OK)
   {
 
     if (opt_silent < 2)
     {
-      (void) put_error(&con, NULL);
+      (void) put_error(con, NULL);
       (void) fflush(stdout);
       return ignore_errors ? -1 : 1;    // Abort
     }
@@ -4212,7 +4222,7 @@
   }
   connected= 1;
 
-  ServerDetect server_detect(&con);
+  ServerDetect server_detect(con);
   server_type= server_detect.getServerType();
 
   build_completion_hash(opt_rehash, 1);
@@ -4238,12 +4248,12 @@
 
   if (connected)
   {
-    tee_fprintf(stdout, _("\nConnection id:\t\t%lu\n"),drizzle_con_thread_id(&con));
+    tee_fprintf(stdout, _("\nConnection id:\t\t%lu\n"),drizzle_con_thread_id(con));
     /*
       Don't remove "limit 1",
       it is protection againts SQL_SELECT_LIMIT=0
     */
-    if (drizzle_query_str(&con,&result,"select DATABASE(), USER() limit 1",
+    if (drizzle_query_str(con, &result, "select DATABASE(), USER() limit 1",
                           &ret) != NULL && ret == DRIZZLE_RETURN_OK &&
         drizzle_result_buffer(&result) == DRIZZLE_RETURN_OK)
     {
@@ -4275,19 +4285,23 @@
   tee_fprintf(stdout, _("Current pager:\t\t%s\n"), pager.c_str());
   tee_fprintf(stdout, _("Using outfile:\t\t'%s'\n"), opt_outfile ? outfile.c_str() : "");
   tee_fprintf(stdout, _("Using delimiter:\t%s\n"), delimiter);
-  tee_fprintf(stdout, _("Server version:\t\t%s\n"), server_version_string(&con));
+  tee_fprintf(stdout, _("Server version:\t\t%s\n"), server_version_string(con));
   tee_fprintf(stdout, _("Protocol:\t\t%s\n"), opt_protocol.c_str());
-  tee_fprintf(stdout, _("Protocol version:\t%d\n"), drizzle_con_protocol_version(&con));
-  tee_fprintf(stdout, _("Connection:\t\t%s\n"), drizzle_con_host(&con));
+  tee_fprintf(stdout, _("Protocol version:\t%d\n"), drizzle_con_protocol_version(con));
+  tee_fprintf(stdout, _("Connection:\t\t%s\n"), drizzle_con_host(con));
 /* XXX need to save this from result
   if ((id= drizzleclient_insert_id(&drizzle)))
     tee_fprintf(stdout, "Insert id:\t\t%s\n", internal::llstr(id, buff));
 */
 
-  if (drizzle_con_uds(&con))
-    tee_fprintf(stdout, _("UNIX socket:\t\t%s\n"), drizzle_con_uds(&con));
+  if (drizzle_con_uds(con))
+  {
+    tee_fprintf(stdout, _("UNIX socket:\t\t%s\n"), drizzle_con_uds(con));
+  }
   else
-    tee_fprintf(stdout, _("TCP port:\t\t%d\n"), drizzle_con_port(&con));
+  {
+    tee_fprintf(stdout, _("TCP port:\t\t%d\n"), drizzle_con_port(con));
+  }
 
   if (safe_updates)
   {
@@ -4426,10 +4440,14 @@
   {
     error= drizzle_result_error(res);
     if (!strcmp(error, ""))
+    {
       error= drizzle_con_error(local_con);
+    }
   }
   else
+  {
     error= drizzle_con_error(local_con);
+  }
 
   return put_info(error, INFO_ERROR,
                   res == NULL ? drizzle_con_error_code(local_con) :
@@ -4580,7 +4598,7 @@
         break;
       case 'v':
         if (connected)
-          processed_prompt->append(drizzle_con_server_version(&con));
+          processed_prompt->append(drizzle_con_server_version(con));
         else
           processed_prompt->append("not_connected");
         break;
@@ -4589,7 +4607,7 @@
         break;
       case 'h':
       {
-        const char *prompt= connected ? drizzle_con_host(&con) : "not_connected";
+        const char *prompt= connected ? drizzle_con_host(con) : "not_connected";
         if (strstr(prompt, "Localhost"))
           processed_prompt->append("localhost");
         else
@@ -4608,13 +4626,13 @@
           break;
         }
 
-        if (drizzle_con_uds(&con))
+        if (drizzle_con_uds(con))
         {
-          const char *pos=strrchr(drizzle_con_uds(&con),'/');
-          processed_prompt->append(pos ? pos+1 : drizzle_con_uds(&con));
+          const char *pos=strrchr(drizzle_con_uds(con),'/');
+          processed_prompt->append(pos ? pos+1 : drizzle_con_uds(con));
         }
         else
-          add_int_to_prompt(drizzle_con_port(&con));
+          add_int_to_prompt(drizzle_con_port(con));
       }
       break;
       case 'U':

=== modified file 'client/drizzledump_data.cc'
--- client/drizzledump_data.cc	2011-11-24 00:05:44 +0000
+++ client/drizzledump_data.cc	2011-11-26 23:14:59 +0000
@@ -543,8 +543,6 @@
   hostName(host),
   drizzleProtocol(drizzle_protocol)
 {
-  drizzle_return_t ret;
-
   if (host.empty())
   {
     host= "localhost";
@@ -564,20 +562,19 @@
     std::cerr << "drizzle_create() failed" << std::endl;
   }
 
-  drizzle_con_create(drizzle, &connection);
-  drizzle_con_set_tcp(&connection, (char *)host.c_str(), port);
-  drizzle_con_set_auth(&connection, (char *)username.c_str(),
-    (char *)password.c_str());
-  drizzle_con_add_options(&connection, 
-    drizzle_protocol ? DRIZZLE_CON_EXPERIMENTAL : DRIZZLE_CON_MYSQL);
-  ret= drizzle_con_connect(&connection);
+  connection= drizzle_con_create(drizzle);
+  drizzle_con_set_tcp(connection, (char *)host.c_str(), port);
+  drizzle_con_set_auth(connection, (char *)username.c_str(), (char *)password.c_str());
+  drizzle_con_add_options(connection, drizzle_protocol ? DRIZZLE_CON_EXPERIMENTAL : DRIZZLE_CON_MYSQL);
+
+  drizzle_return_t ret= drizzle_con_connect(connection);
   if (ret != DRIZZLE_RETURN_OK)
   {
     errorHandler(NULL, ret, "when trying to connect");
     throw std::exception();
   }
 
-  ServerDetect server_detect= ServerDetect(&connection);
+  ServerDetect server_detect= ServerDetect(connection);
 
   serverType= server_detect.getServerType();
   serverVersion= server_detect.getServerVersion();
@@ -587,7 +584,7 @@
 {
   drizzle_return_t ret;
   drizzle_result_st* result= new drizzle_result_st;
-  if (drizzle_query_str(&connection, result, str_query.c_str(), &ret) == NULL ||
+  if (drizzle_query_str(connection, result, str_query.c_str(), &ret) == NULL or
       ret != DRIZZLE_RETURN_OK)
   {
     if (ret == DRIZZLE_RETURN_ERROR_CODE)
@@ -598,16 +595,14 @@
     }
     else
     {
-      std::cerr << _("Error executing query: ") <<
-        drizzle_con_error(&connection) << std::endl;
+      std::cerr << _("Error executing query: ") << drizzle_con_error(connection) << std::endl;
     }
     return NULL;
   }
 
   if (drizzle_result_buffer(result) != DRIZZLE_RETURN_OK)
   {
-    std::cerr << _("Could not buffer result: ") <<
-        drizzle_con_error(&connection) << std::endl;
+    std::cerr << _("Could not buffer result: ") << drizzle_con_error(connection) << std::endl;
     return NULL;
   }
   return result;
@@ -624,7 +619,7 @@
   drizzle_return_t ret;
   drizzle_result_st result;
 
-  if (drizzle_query_str(&connection, &result, str_query.c_str(), &ret) == NULL ||
+  if (drizzle_query_str(connection, &result, str_query.c_str(), &ret) == NULL or
       ret != DRIZZLE_RETURN_OK)
   {
     if (ret == DRIZZLE_RETURN_ERROR_CODE)
@@ -635,8 +630,7 @@
     }
     else
     {
-      std::cerr << _("Error executing query: ") <<
-        drizzle_con_error(&connection) << std::endl;
+      std::cerr << _("Error executing query: ") << drizzle_con_error(connection) << std::endl;
     }
     return false;
   }
@@ -649,15 +643,20 @@
 {
   drizzle_return_t ret;
   drizzle_result_st result;
-  if (drizzle_select_db(&connection, &result, databaseName.c_str(), &ret) == 
-    NULL || ret != DRIZZLE_RETURN_OK)
+
+  if (drizzle_select_db(connection, &result, databaseName.c_str(), &ret) == NULL or 
+      ret != DRIZZLE_RETURN_OK)
   {
     std::cerr << _("Error: Could not set db '") << databaseName << "'" << std::endl;
     if (ret == DRIZZLE_RETURN_ERROR_CODE)
+    {
       drizzle_result_free(&result);
+    }
+
     return false;
   }
   drizzle_result_free(&result);
+
   return true;
 }
 
@@ -666,8 +665,7 @@
 {
   if (res == NULL)
   {
-    std::cerr << _("Got error: ") << drizzle_con_error(&connection) << " "
-      << when << std::endl;
+    std::cerr << _("Got error: ") << drizzle_con_error(connection) << " " << when << std::endl;
   }
   else if (ret == DRIZZLE_RETURN_ERROR_CODE)
   {
@@ -690,6 +688,6 @@
     std::cerr << _("-- Disconnecting from ") << hostName << "..." << std::endl;
   }
 
-  drizzle_con_free(&connection);
+  drizzle_con_free(connection);
   drizzle_free(drizzle);
 }

=== modified file 'client/drizzledump_data.h'
--- client/drizzledump_data.h	2011-11-24 00:05:44 +0000
+++ client/drizzledump_data.h	2011-11-26 23:14:59 +0000
@@ -213,16 +213,21 @@
 {
   private:
     drizzle_st *drizzle;
-    drizzle_con_st connection;
+    drizzle_con_st *connection;
     std::string hostName;
     bool drizzleProtocol;
     ServerDetect::server_type serverType;
     std::string serverVersion;
 
   public:
-    DrizzleDumpConnection(std::string &host, uint16_t port,
-      std::string &username, std::string &password, bool drizzle_protocol);
+    DrizzleDumpConnection(std::string &host,
+                          uint16_t port,
+                          std::string &username,
+                          std::string &password,
+                          bool drizzle_protocol);
+
     ~DrizzleDumpConnection();
+
     void errorHandler(drizzle_result_st *res,  drizzle_return_t ret, const char *when);
     drizzle_result_st* query(std::string &str_query);
     bool queryNoResult(std::string &str_query);
@@ -232,6 +237,7 @@
       std::string str_query(ch_query);
       return query(str_query);
     }
+
     bool queryNoResult(const char* ch_query)
     {
       std::string str_query(ch_query);
@@ -263,7 +269,9 @@
     void writeString(std::string &str)
     {
       if (not connection->queryNoResult(str))
+      {
         throw std::exception();
+      }
     }
 
     void setConnection(DrizzleDumpConnection *conn) { connection= conn; }

=== modified file 'client/drizzleimport.cc'
--- client/drizzleimport.cc	2011-11-24 00:05:44 +0000
+++ client/drizzleimport.cc	2011-11-26 23:14:59 +0000
@@ -210,8 +210,11 @@
     return 0;
   }
 
-  if (!(con= drizzle_con_add_tcp(drizzle,NULL,(char *)host.c_str(),opt_drizzle_port,(char *)user.c_str(),(char *)passwd.c_str(),
-                                 (char *)database.c_str(), use_drizzle_protocol ? DRIZZLE_CON_EXPERIMENTAL : DRIZZLE_CON_MYSQL)))
+  if (!(con= drizzle_con_add_tcp(drizzle,
+                                 host.c_str(), opt_drizzle_port,
+                                 user.c_str(), passwd.c_str(),
+                                 database.c_str(),
+                                 use_drizzle_protocol ? DRIZZLE_CON_EXPERIMENTAL : DRIZZLE_CON_MYSQL)))
   {
     return 0;
   }

=== modified file 'client/drizzleslap.cc'
--- client/drizzleslap.cc	2011-11-24 00:05:44 +0000
+++ client/drizzleslap.cc	2011-11-26 23:14:59 +0000
@@ -233,8 +233,8 @@
 void option_cleanup(OptionString *stmt);
 void concurrency_loop(drizzle_con_st &con, uint32_t current, OptionString *eptr);
 static void run_statements(drizzle_con_st &con, Statement *stmt);
-void slap_connect(drizzle_con_st &con, bool connect_to_schema);
-void slap_close(drizzle_con_st &con);
+drizzle_con_st *slap_connect(bool connect_to_schema);
+void slap_close(drizzle_con_st *con);
 static int run_query(drizzle_con_st &con, drizzle_result_st *result, const char *query, int len);
 void standard_deviation(Conclusions &con, Stats *sptr);
 
@@ -273,23 +273,25 @@
   uint64_t counter= 0, queries;
   uint64_t detach_counter;
   uint32_t commit_counter;
-  boost::scoped_ptr<drizzle_con_st> con_ap(new drizzle_con_st);
-  drizzle_con_st &con= *con_ap.get();
   drizzle_result_st result;
   drizzle_row_t row;
   Statement *ptr;
 
   master_wakeup.wait();
 
-  slap_connect(con, true);
+  drizzle_con_st *con= slap_connect(true);
 
   if (verbose >= 3)
+  {
     printf("connected!\n");
+  }
   queries= 0;
 
   commit_counter= 0;
   if (commit_rate)
-    run_query(con, NULL, "SET AUTOCOMMIT=0", strlen("SET AUTOCOMMIT=0"));
+  {
+    run_query(*con, NULL, "SET AUTOCOMMIT=0", strlen("SET AUTOCOMMIT=0"));
+  }
 
 limit_not_met:
   for (ptr= ctx->getStmt(), detach_counter= 0;
@@ -299,7 +301,7 @@
     if (not opt_only_print && detach_rate && !(detach_counter % detach_rate))
     {
       slap_close(con);
-      slap_connect(con, true);
+      con= slap_connect(true);
     }
 
     /*
@@ -329,7 +331,7 @@
 
         int length= snprintf(buffer, HUGE_STRING_LENGTH, "%.*s '%s'", (int)ptr->getLength(), ptr->getString(), key);
 
-        if (run_query(con, &result, buffer, length))
+        if (run_query(*con, &result, buffer, length))
         {
           if ((ptr->getType() == UPDATE_TYPE_REQUIRES_PREFIX) and commit_rate)
           {
@@ -342,7 +344,7 @@
           else
           {
             fprintf(stderr,"%s: Cannot run query %.*s ERROR : %s\n",
-                    SLAP_NAME, (uint32_t)length, buffer, drizzle_con_error(&con));
+                    SLAP_NAME, (uint32_t)length, buffer, drizzle_con_error(con));
             abort();
           }
         }
@@ -350,7 +352,7 @@
     }
     else
     {
-      if (run_query(con, &result, ptr->getString(), ptr->getLength()))
+      if (run_query(*con, &result, ptr->getString(), ptr->getLength()))
       {
         if ((ptr->getType() == UPDATE_TYPE_REQUIRES_PREFIX) and commit_rate)
         {
@@ -363,7 +365,7 @@
         else
         {
           fprintf(stderr,"%s: Cannot run query %.*s ERROR : %s\n",
-                  SLAP_NAME, (uint32_t)ptr->getLength(), ptr->getString(), drizzle_con_error(&con));
+                  SLAP_NAME, (uint32_t)ptr->getLength(), ptr->getString(), drizzle_con_error(con));
           abort();
         }
       }
@@ -380,7 +382,7 @@
     if (commit_rate && (++commit_counter == commit_rate) and not is_failed_update)
     {
       commit_counter= 0;
-      run_query(con, NULL, "COMMIT", strlen("COMMIT"));
+      run_query(*con, NULL, "COMMIT", strlen("COMMIT"));
     }
 
     /* If the timer is set, and the alarm is not active then end */
@@ -401,9 +403,12 @@
 
 end:
   if (commit_rate)
-    run_query(con, NULL, "COMMIT", strlen("COMMIT"));
+  {
+    run_query(*con, NULL, "COMMIT", strlen("COMMIT"));
+  }
 
   slap_close(con);
+  con= NULL;
 
   delete ctx;
 }
@@ -560,7 +565,6 @@
 
     uint64_t temp_drizzle_port= 0;
     boost::scoped_ptr<drizzle_con_st> con_ap(new drizzle_con_st);
-    drizzle_con_st &con= *con_ap.get();
     OptionString *eptr;
 
     // Disable allow_guessing
@@ -674,7 +678,7 @@
     /* globals? Yes, so we only have to run strlen once */
     delimiter_length= delimiter.length();
 
-    slap_connect(con, false);
+    drizzle_con_st *con= slap_connect(false);
 
     /* Main iterations loop */
 burnin:
@@ -690,24 +694,26 @@
       if (concurrency.size())
       {
         for (current= &concurrency[0]; current && *current; current++)
-          concurrency_loop(con, *current, eptr);
+          concurrency_loop(*con, *current, eptr);
       }
       else
       {
         uint32_t infinite= 1;
         do {
-          concurrency_loop(con, infinite, eptr);
+          concurrency_loop(*con, infinite, eptr);
         }
         while (infinite++);
       }
 
       if (not opt_preserve)
-        drop_schema(con, create_schema_string.c_str());
+        drop_schema(*con, create_schema_string.c_str());
 
     } while (eptr ? (eptr= eptr->getNext()) : 0);
 
     if (opt_burnin)
+    {
       goto burnin;
+    }
 
     slap_close(con);
 
@@ -2372,12 +2378,12 @@
   }
 }
 
-void slap_close(drizzle_con_st &con)
+void slap_close(drizzle_con_st *con)
 {
-  drizzle_free(drizzle_con_drizzle(&con));
+  drizzle_free(drizzle_con_drizzle(con));
 }
 
-void slap_connect(drizzle_con_st &con, bool connect_to_schema)
+drizzle_con_st* slap_connect(bool connect_to_schema)
 {
   /* Connect to server */
   static uint32_t connection_retry_sleep= 100000; /* Microseconds */
@@ -2388,12 +2394,13 @@
   if (opt_delayed_start)
     usleep(random()%opt_delayed_start);
 
+  drizzle_con_st* con;
   if ((drizzle= drizzle_create()) == NULL or
-      drizzle_con_add_tcp(drizzle, &con, host.c_str(), opt_drizzle_port,
-        user.c_str(),
-        opt_password.c_str(),
-        connect_to_schema ? create_schema_string.c_str() : NULL,
-        use_drizzle_protocol ? DRIZZLE_CON_EXPERIMENTAL : DRIZZLE_CON_MYSQL) == NULL)
+      (con= drizzle_con_add_tcp(drizzle,
+                                host.c_str(), opt_drizzle_port,
+                                user.c_str(), opt_password.c_str(),
+                                connect_to_schema ? create_schema_string.c_str() : NULL,
+                                use_drizzle_protocol ? DRIZZLE_CON_EXPERIMENTAL : DRIZZLE_CON_MYSQL)) == NULL)
   {
     fprintf(stderr,"%s: Error creating drizzle object\n", SLAP_NAME);
     abort();
@@ -2403,12 +2410,12 @@
 
   if (opt_only_print)
   {
-    return;
+    return con;
   }
 
   for (uint32_t x= 0; x < 10; x++)
   {
-    if ((ret= drizzle_con_connect(&con)) == DRIZZLE_RETURN_OK)
+    if ((ret= drizzle_con_connect(con)) == DRIZZLE_RETURN_OK)
     {
       /* Connect suceeded */
       connect_error= 0;
@@ -2419,9 +2426,11 @@
   if (connect_error)
   {
     fprintf(stderr,"%s: Error when connecting to server: %d %s\n", SLAP_NAME,
-            ret, drizzle_con_error(&con));
+            ret, drizzle_con_error(con));
     abort();
   }
+
+  return con;
 }
 
 void standard_deviation(Conclusions &con, Stats *sptr)

=== modified file 'examples/client.cc'
--- examples/client.cc	2011-11-24 03:44:17 +0000
+++ examples/client.cc	2011-11-26 23:14:59 +0000
@@ -100,7 +100,6 @@
   drizzle_return_t ret;
   uint32_t x;
   int wait_for_connections= 0;
-  drizzle_con_st *con;
   client_con_st *client_con;
   char *host= NULL;
   in_port_t port= 0;
@@ -221,12 +220,11 @@
   for (x= 0; x < client.client_con_count; x++)
   {
     /* This may fail if there is other initialization that fails. See docs. */
-    con= drizzle_con_add_tcp(client.drizzle,
-                              &(client.client_con_list[x].con),
-                              host, port, user, password, db,
-                              client.mysql_protocol
-                              ? DRIZZLE_CON_MYSQL
-                              : DRIZZLE_CON_NONE);
+    drizzle_con_st *con= drizzle_con_add_tcp(client.drizzle,
+                                             host, port, user, password, db,
+                                             client.mysql_protocol
+                                             ? DRIZZLE_CON_MYSQL
+                                             : DRIZZLE_CON_NONE);
     if (con == NULL)
     {
       CLIENT_ERROR("drizzle_con_add_tcp", 0, &client);
@@ -248,6 +246,7 @@
       CLIENT_ERROR("drizzle_con_wait", ret, &client);
     }
 
+    drizzle_con_st* con;
     while ((con= drizzle_con_ready(client.drizzle)) != NULL)
     {
       client_con= (client_con_st *)drizzle_con_context(con);

=== modified file 'examples/pipe_query.cc'
--- examples/pipe_query.cc	2011-11-24 00:05:44 +0000
+++ examples/pipe_query.cc	2011-11-26 23:14:59 +0000
@@ -55,20 +55,11 @@
   size_t buffer_size= 0;
   ssize_t read_size= 0;
   drizzle_st *drizzle;
-  drizzle_con_st *con= (drizzle_con_st*)malloc(sizeof(drizzle_con_st));
   drizzle_result_st result;
-  drizzle_return_t ret;
-  drizzle_field_t field;
   size_t offset;
   size_t size;
   size_t total;
 
-  if (con == NULL)
-  {
-    printf("Failed to allocate memory for drizzle connection");
-    exit(1);
-  }
-
   /* The docs say this might fail, so check for errors. */
   if ((drizzle= drizzle_create()) == NULL)
   {
@@ -76,7 +67,8 @@
     exit(1);
   }
 
-  if (drizzle_con_create(drizzle, con) == NULL)
+  drizzle_con_st *con;
+  if ((con= drizzle_con_create(drizzle)) == NULL)
   {
     printf("drizzle_con_create:%s\n", drizzle_error(drizzle));
     exit(1);
@@ -146,6 +138,7 @@
 
   } while ((read_size= read(0, buffer + buffer_size, BUFFER_CHUNK)) != 0);
 
+  drizzle_return_t ret;
   (void)drizzle_query(con, &result, buffer, buffer_size, &ret);
   if (ret != DRIZZLE_RETURN_OK)
   {
@@ -166,9 +159,11 @@
   {
     while (1)
     {
-      field= drizzle_field_read(&result, &offset, &size, &total, &ret);
+      drizzle_field_t field= drizzle_field_read(&result, &offset, &size, &total, &ret);
       if (ret == DRIZZLE_RETURN_ROW_END)
+      {
         break;
+      }
       else if (ret != DRIZZLE_RETURN_OK)
       {
         printf("drizzle_field_read:%s\n", drizzle_error(drizzle));
@@ -176,12 +171,18 @@
       }
 
       if (field == NULL)
+      {
         printf("NULL");
+      }
       else
+      {
         printf("%.*s", (int)size, field);
+      }
 
       if (offset + size == total)
+      {
         printf("\t");
+      }
     }
 
     printf("\n");
@@ -197,6 +198,5 @@
   drizzle_con_free(con);
   drizzle_free(drizzle);
 
-  free(con);
   return 0;
 }

=== modified file 'examples/proxy.cc'
--- examples/proxy.cc	2011-11-24 03:44:17 +0000
+++ examples/proxy.cc	2011-11-26 23:14:59 +0000
@@ -71,9 +71,6 @@
   drizzle_verbose_t verbose= DRIZZLE_VERBOSE_NEVER;
   drizzle_return_t ret;
   drizzle_st *drizzle;
-  drizzle_con_st *con_listen= (drizzle_con_st*)malloc(sizeof(drizzle_con_st));
-  drizzle_con_st *server= (drizzle_con_st*)malloc(sizeof(drizzle_con_st));
-  drizzle_con_st *client= (drizzle_con_st*)malloc(sizeof(drizzle_con_st));
   drizzle_result_st server_result;
   drizzle_result_st client_result;
   drizzle_column_st column;
@@ -158,7 +155,8 @@
   drizzle_set_option(drizzle, DRIZZLE_FREE_OBJECTS, true);
   drizzle_set_verbose(drizzle, verbose);
 
-  if (drizzle_con_create(drizzle, con_listen) == NULL)
+  drizzle_con_st *con_listen;
+  if ((con_listen= drizzle_con_create(drizzle)) == NULL)
   {
     fprintf(stderr, "drizzle_con_create:NULL\n");
     return 1;
@@ -180,14 +178,15 @@
 
   while (1)
   {
-    (void)drizzle_con_accept(drizzle, server, &ret);
+    drizzle_con_st *server= drizzle_con_accept(drizzle, &ret);
     if (ret != DRIZZLE_RETURN_OK)
     {
       fprintf(stderr, "drizzle_con_accept:%s\n", drizzle_error(drizzle));
       return 1;
     }
 
-    if (drizzle_con_create(drizzle, client) == NULL)
+    drizzle_con_st *client;
+    if ((client= drizzle_con_create(drizzle)) == NULL)
     {
       fprintf(stderr, "drizzle_con_create:NULL\n");
       return 1;
@@ -196,7 +195,9 @@
     drizzle_con_add_options(client,
                             DRIZZLE_CON_RAW_PACKET | DRIZZLE_CON_RAW_SCRAMBLE);
     if (client_mysql)
+    {
       drizzle_con_add_options(client, DRIZZLE_CON_MYSQL);
+    }
     drizzle_con_set_tcp(client, client_host, client_port);
 
     ret= drizzle_con_connect(client);
@@ -223,10 +224,6 @@
   drizzle_con_free(con_listen);
   drizzle_free(drizzle);
 
-  free(con_listen);
-  free(server);
-  free(client);
-
   return 0;
 }
 

=== modified file 'examples/server.cc'
--- examples/server.cc	2011-11-24 03:44:17 +0000
+++ examples/server.cc	2011-11-26 23:14:59 +0000
@@ -71,8 +71,6 @@
   drizzle_verbose_t verbose= DRIZZLE_VERBOSE_NEVER;
   drizzle_return_t ret;
   drizzle_st *drizzle;
-  drizzle_con_st *con_listen= (drizzle_con_st*)malloc(sizeof(drizzle_con_st));
-  drizzle_con_st *con= (drizzle_con_st*)malloc(sizeof(drizzle_con_st));
   drizzle_result_st result;
   drizzle_column_st column;
 
@@ -141,7 +139,8 @@
   drizzle_set_option(drizzle, DRIZZLE_FREE_OBJECTS, true);
   drizzle_set_verbose(drizzle, verbose);
 
-  if (drizzle_con_create(drizzle, con_listen) == NULL)
+  drizzle_con_st* con_listen;
+  if ((con_listen= drizzle_con_create(drizzle)) == NULL)
   {
     fprintf(stderr, "drizzle_con_create:NULL\n");
     return 1;
@@ -163,7 +162,7 @@
 
   while (1)
   {
-    (void)drizzle_con_accept(drizzle, con, &ret);
+    drizzle_con_st *con= drizzle_con_accept(drizzle, &ret);
     if (ret != DRIZZLE_RETURN_OK)
     {
       fprintf(stderr, "drizzle_con_accept:%s\n", drizzle_error(drizzle));
@@ -186,9 +185,6 @@
   drizzle_con_free(con_listen);
   drizzle_free(drizzle);
 
-  free(con);
-  free(con_listen);
-
   return 0;
 }
 

=== modified file 'examples/simple_multi.cc'
--- examples/simple_multi.cc	2011-11-24 00:05:44 +0000
+++ examples/simple_multi.cc	2011-11-26 23:14:59 +0000
@@ -34,9 +34,9 @@
  *
  */
 
-#include <stdio.h>
-#include <string.h>
-#include <stdlib.h>
+#include <cstdio>
+#include <cstring>
+#include <cstdlib>
 
 #include <libdrizzle-2.0/drizzle_client.h>
 
@@ -46,85 +46,90 @@
 {
   const char *query= "SELECT table_schema,table_name FROM tables";
   drizzle_st *drizzle;
-  drizzle_con_st *con;
-  drizzle_result_st *result;
-  drizzle_query_st *ql;
-  drizzle_return_t ret;
-  drizzle_row_t row;
-  int x;
 
-  con= (drizzle_con_st*)malloc(sizeof(drizzle_con_st) * SIMPLE_MULTI_COUNT);
-  result= (drizzle_result_st*)malloc(sizeof(drizzle_result_st) * SIMPLE_MULTI_COUNT);
-  ql= (drizzle_query_st*)malloc(sizeof(drizzle_query_st) * SIMPLE_MULTI_COUNT);
+  std::vector<drizzle_con_st *> cons;
+  std::vector<drizzle_result_st *> result;
+  std::vector<drizzle_query_st *> ql;
 
   if ((drizzle= drizzle_create()) == NULL)
   {
-    printf("drizzle_create:NULL\n");
+    fprintf(stderr, "drizzle_create:NULL\n");
     return 1;
   }
 
   /* Create SIMPLE_MULTI_COUNT connections and initialize query list. */
-  for (x= 0; x < SIMPLE_MULTI_COUNT; x++)
+  for (int x= 0; x < SIMPLE_MULTI_COUNT; x++)
   {
+    drizzle_con_st *con;
     if (x == 0)
     {
-      if (drizzle_con_create(drizzle, &(con[0])) == NULL)
+      if ((con= drizzle_con_create(drizzle)) == NULL)
       {
         printf("drizzle_con_create:%s\n", drizzle_error(drizzle));
         return 1;
       }
 
       if (argc == 2 && !strcmp(argv[1], "-m"))
-        drizzle_con_add_options(&(con[0]), DRIZZLE_CON_MYSQL);
+      {
+        drizzle_con_add_options(con, DRIZZLE_CON_MYSQL);
+      }
       else if (argc != 1)
       {
         printf("usage: %s [-m]\n", argv[0]);
         return 1;
       }
 
-      drizzle_con_set_db(&(con[0]), "information_schema");
+      drizzle_con_set_db(con, "information_schema");
+      cons.push_back(con);
     }
     else
     {
-      if (drizzle_con_clone(drizzle, &(con[x]), &(con[0])) == NULL)
+      if ((con= drizzle_con_clone(drizzle, cons[0])) == NULL)
       {
-        printf("drizzle_con_clone:%s\n", drizzle_error(drizzle));
+        fprintf(stderr, "drizzle_con_clone:%s\n", drizzle_error(drizzle));
         return 1;
       }
     }
 
-    if (drizzle_query_add(drizzle, &(ql[x]), &(con[x]), &(result[x]), query,
-                          strlen(query), DRIZZLE_QUERY_NONE, NULL) == NULL)
+    cons.push_back(con);
+    drizzle_result_st *res= drizzle_result_create(con, NULL);
+    result.push_back(res);
+
+    drizzle_query_st *exec_query;
+    if ((exec_query= drizzle_query_add(drizzle, NULL, con, res, query, strlen(query), DRIZZLE_QUERY_NONE, NULL)) == NULL)
     {
-      printf("drizzle_query_add:%s\n", drizzle_error(drizzle));
+      fprintf(stderr, "drizzle_query_add:%s\n", drizzle_error(drizzle));
       return 1;
     }
+    ql.push_back(exec_query);
   }
 
-  ret= drizzle_query_run_all(drizzle);
+  drizzle_return_t ret= drizzle_query_run_all(drizzle);
   if (ret != DRIZZLE_RETURN_OK)
   {
     printf("drizzle_query_run_all:%s\n", drizzle_error(drizzle));
     return 1;
   }
 
-  for (x= 0; x < SIMPLE_MULTI_COUNT; x++)
+  uint32_t x= 0;
+  for (std::vector<drizzle_result_st *>::iterator iter= result.begin(); iter != result.end(); iter++)
   {
-    if (drizzle_result_error_code(&(result[x])) != 0)
+    if (drizzle_result_error_code(*iter) != 0)
     {
-      printf("%d:%s\n", drizzle_result_error_code(&(result[x])),
-             drizzle_result_error(&(result[x])));
+      printf("%d:%s\n", drizzle_result_error_code(*iter),
+             drizzle_result_error(*iter));
       continue;
     }
 
-    while ((row= drizzle_row_next(&(result[x]))) != NULL)
-      printf("%d %s:%s\n", x, row[0], row[1]);
+    drizzle_row_t row;
+    while ((row= drizzle_row_next(*iter)) != NULL)
+    {
+      printf("%x %s:%s\n", x, row[0], row[1]);
+    }
+    x++;
   }
 
   drizzle_free(drizzle);
 
-  free(con);
-  free(result);
-  free(ql);
   return 0;
 }

=== modified file 'examples/sqlite_server.cc'
--- examples/sqlite_server.cc	2011-11-24 03:44:17 +0000
+++ examples/sqlite_server.cc	2011-11-26 23:14:59 +0000
@@ -72,7 +72,7 @@
 typedef struct
 {
   drizzle_st *drizzle;
-  drizzle_con_st con;
+  drizzle_con_st *con;
   drizzle_result_st result;
   drizzle_column_st column;
   sqlite3* db;
@@ -95,7 +95,6 @@
   in_port_t port= 0;
   drizzle_return_t ret;
   sqlite_server *server= (sqlite_server*)malloc(sizeof(sqlite_server));
-  drizzle_con_st *con_listen= (drizzle_con_st*)malloc(sizeof(drizzle_con_st));
 
   server->db= NULL;
   server->verbose= DRIZZLE_VERBOSE_NEVER;
@@ -172,7 +171,8 @@
   drizzle_set_option(server->drizzle, DRIZZLE_FREE_OBJECTS, true);
   drizzle_set_verbose(server->drizzle, server->verbose);
 
-  if (drizzle_con_create(server->drizzle, con_listen) == NULL)
+  drizzle_con_st *con_listen;
+  if ((con_listen= drizzle_con_create(server->drizzle)) == NULL)
   {
     fprintf(stderr, "drizzle_con_create:NULL\n");
     return 1;
@@ -182,7 +182,9 @@
   drizzle_con_set_tcp(con_listen, host, port);
 
   if (mysql)
+  {
     drizzle_con_add_options(con_listen, DRIZZLE_CON_MYSQL);
+  }
 
   if (drizzle_con_listen(con_listen) != DRIZZLE_RETURN_OK)
   {
@@ -192,7 +194,7 @@
 
   while (1)
   {
-    (void)drizzle_con_accept(server->drizzle, &(server->con), &ret);
+    server->con= drizzle_con_accept(server->drizzle, &ret);
     if (ret != DRIZZLE_RETURN_OK)
     {
       fprintf(stderr, "drizzle_con_accept:%s\n", drizzle_error(server->drizzle));
@@ -201,7 +203,7 @@
 
     server_run(server);
 
-    drizzle_con_free(&(server->con));
+    drizzle_con_free(server->con);
 
     if (count > 0)
     {
@@ -215,7 +217,6 @@
   drizzle_con_free(con_listen);
   drizzle_free(server->drizzle);
   sqlite3_close(server->db);
-  free(con_listen);
   free(server);
 
   return 0;
@@ -231,26 +232,25 @@
   char *sqlite_err;
 
   /* Handshake packets. */
-  drizzle_con_set_protocol_version(&(server->con), 10);
-  drizzle_con_set_server_version(&(server->con), "libdrizzle+SQLite");
-  drizzle_con_set_thread_id(&(server->con), 1);
-  drizzle_con_set_scramble(&(server->con),
-                           (const uint8_t *)"ABCDEFGHIJKLMNOPQRST");
-  drizzle_con_set_capabilities(&(server->con), DRIZZLE_CAPABILITIES_NONE);
-  drizzle_con_set_charset(&(server->con), 8);
-  drizzle_con_set_status(&(server->con), DRIZZLE_CON_STATUS_NONE);
-  drizzle_con_set_max_packet_size(&(server->con), DRIZZLE_MAX_PACKET_SIZE);
+  drizzle_con_set_protocol_version(server->con, 10);
+  drizzle_con_set_server_version(server->con, "libdrizzle+SQLite");
+  drizzle_con_set_thread_id(server->con, 1);
+  drizzle_con_set_scramble(server->con, (const uint8_t *)"ABCDEFGHIJKLMNOPQRST");
+  drizzle_con_set_capabilities(server->con, DRIZZLE_CAPABILITIES_NONE);
+  drizzle_con_set_charset(server->con, 8);
+  drizzle_con_set_status(server->con, DRIZZLE_CON_STATUS_NONE);
+  drizzle_con_set_max_packet_size(server->con, DRIZZLE_MAX_PACKET_SIZE);
 
-  ret= drizzle_handshake_server_write(&(server->con));
+  ret= drizzle_handshake_server_write(server->con);
   DRIZZLE_RETURN_CHECK(ret, "drizzle_handshake_server_write", server->drizzle)
 
-  ret= drizzle_handshake_client_read(&(server->con));
+  ret= drizzle_handshake_client_read(server->con);
   DRIZZLE_RETURN_CHECK(ret, "drizzle_handshake_client_read", server->drizzle)
 
-  if (drizzle_result_create(&(server->con), &(server->result)) == NULL)
+  if (drizzle_result_create(server->con, &(server->result)) == NULL)
     DRIZZLE_RETURN_ERROR("drizzle_result_create", server->drizzle)
 
-  ret= drizzle_result_write(&(server->con), &(server->result), true);
+  ret= drizzle_result_write(server->con, &(server->result), true);
   DRIZZLE_RETURN_CHECK(ret, "drizzle_result_write", server->drizzle)
 
   /* Command loop. */
@@ -259,7 +259,7 @@
     drizzle_result_free(&(server->result));
     free(data);
 
-    data= (uint8_t *)drizzle_con_command_buffer(&(server->con), &command, &total, &ret);
+    data= (uint8_t *)drizzle_con_command_buffer(server->con, &command, &total, &ret);
     if (ret == DRIZZLE_RETURN_LOST_CONNECTION ||
         (ret == DRIZZLE_RETURN_OK && command == DRIZZLE_COMMAND_QUIT))
     {
@@ -274,19 +274,19 @@
              data == NULL ? "NULL" : (char *)data);
     }
 
-    if (drizzle_result_create(&(server->con), &(server->result)) == NULL)
+    if (drizzle_result_create(server->con, &(server->result)) == NULL)
       DRIZZLE_RETURN_ERROR("drizzle_result_create", server->drizzle)
 
     if (command != DRIZZLE_COMMAND_QUERY ||
         !strcasecmp((char *)data, "SHOW DATABASES"))
     {
-      ret= drizzle_result_write(&(server->con), &(server->result), true);
+      ret= drizzle_result_write(server->con, &(server->result), true);
       DRIZZLE_RETURN_CHECK(ret, "drizzle_result_write", server->drizzle)
 
       if (command == DRIZZLE_COMMAND_FIELD_LIST)
       {
         drizzle_result_set_eof(&(server->result), true);
-        ret= drizzle_result_write(&(server->con), &(server->result), true);
+        ret= drizzle_result_write(server->con, &(server->result), true);
         DRIZZLE_RETURN_CHECK(ret, "drizzle_result_write", server->drizzle)
       }
 
@@ -327,7 +327,7 @@
       {
         drizzle_result_set_error_code(&(server->result), (uint16_t)sqlite_ret);
         drizzle_result_set_error(&(server->result), sqlite_err);
-        ret= drizzle_result_write(&(server->con), &(server->result), true);
+        ret= drizzle_result_write(server->con, &(server->result), true);
         DRIZZLE_RETURN_CHECK(ret, "drizzle_result_write", server->drizzle)
 
         fprintf(stderr, "sqlite3_exec:%s\n", sqlite_err);
@@ -340,13 +340,13 @@
     if (server->rows == 0)
     {
       drizzle_result_set_column_count(&(server->result), 0);
-      ret= drizzle_result_write(&(server->con), &(server->result), true);
+      ret= drizzle_result_write(server->con, &(server->result), true);
       DRIZZLE_RETURN_CHECK(ret, "drizzle_result_write", server->drizzle)
     }
     else
     {
       drizzle_result_set_eof(&(server->result), true);
-      ret= drizzle_result_write(&(server->con), &(server->result), true);
+      ret= drizzle_result_write(server->con, &(server->result), true);
       DRIZZLE_RETURN_CHECK(ret, "drizzle_result_write", server->drizzle)
     }
   }
@@ -364,7 +364,7 @@
     server->send_columns= false;
     drizzle_result_set_column_count(&(server->result), (uint16_t)field_count);
 
-    ret= drizzle_result_write(&(server->con), &(server->result), false);
+    ret= drizzle_result_write(server->con, &(server->result), false);
     DRIZZLE_RETURN_CHECK_VAL(ret, "drizzle_result_write", server->drizzle)
 
     if (drizzle_column_create(&(server->result), &(server->column)) == NULL)
@@ -395,7 +395,7 @@
 
     drizzle_result_set_eof(&(server->result), true);
 
-    ret= drizzle_result_write(&(server->con), &(server->result), false);
+    ret= drizzle_result_write(server->con, &(server->result), false);
     DRIZZLE_RETURN_CHECK_VAL(ret, "drizzle_result_write", server->drizzle)
   }
 
@@ -439,7 +439,7 @@
 
   drizzle_result_set_column_count(&(server->result), 1);
 
-  ret= drizzle_result_write(&(server->con), &(server->result), false);
+  ret= drizzle_result_write(server->con, &(server->result), false);
   DRIZZLE_RETURN_CHECK_VAL(ret, "drizzle_result_write", server->drizzle)
 
   if (drizzle_column_create(&(server->result), &(server->column)) == NULL)
@@ -464,7 +464,7 @@
 
   drizzle_result_set_eof(&(server->result), true);
 
-  ret= drizzle_result_write(&(server->con), &(server->result), false);
+  ret= drizzle_result_write(server->con, &(server->result), false);
   DRIZZLE_RETURN_CHECK_VAL(ret, "drizzle_result_write", server->drizzle)
 
   /* This is needed for MySQL and old Drizzle protocol. */
@@ -476,7 +476,7 @@
   ret= drizzle_field_write(&(server->result), fields[0], sizes[0], sizes[0]);
   DRIZZLE_RETURN_CHECK_VAL(ret, "drizzle_field_write", server->drizzle)
 
-  ret= drizzle_result_write(&(server->con), &(server->result), true);
+  ret= drizzle_result_write(server->con, &(server->result), true);
   DRIZZLE_RETURN_CHECK_VAL(ret, "drizzle_result_write", server->drizzle)
 
   return DRIZZLE_RETURN_OK;

=== modified file 'libdrizzle-2.0/conn.cc'
--- libdrizzle-2.0/conn.cc	2011-11-18 16:11:02 +0000
+++ libdrizzle-2.0/conn.cc	2011-11-26 23:14:59 +0000
@@ -176,34 +176,40 @@
   return con->options;
 }
 
-void drizzle_con_set_options(drizzle_con_st *con,
-                             int options)
+void drizzle_con_set_options(drizzle_con_st *con, int options)
 {
   con->options= options;
 }
 
-void drizzle_con_add_options(drizzle_con_st *con,
-                             int options)
+void drizzle_con_add_options(drizzle_con_st *con, int options)
 {
   con->options|= options;
 
   /* If asking for the experimental Drizzle protocol, clean the MySQL flag. */
   if (con->options & DRIZZLE_CON_EXPERIMENTAL)
+  {
     con->options&= ~DRIZZLE_CON_MYSQL;
+  }
 }
 
-void drizzle_con_remove_options(drizzle_con_st *con,
-                                int options)
+void drizzle_con_remove_options(drizzle_con_st *con, int options)
 {
   con->options&= ~options;
 }
 
 const char *drizzle_con_host(const drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return NULL;
+  }
+
   if (con->socket_type == DRIZZLE_CON_SOCKET_TCP)
   {
     if (con->socket.tcp.host == NULL && !(con->options & DRIZZLE_CON_LISTEN))
+    {
       return DRIZZLE_DEFAULT_TCP_HOST;
+    }
 
     return con->socket.tcp.host;
   }
@@ -234,7 +240,9 @@
   con->socket_type= DRIZZLE_CON_SOCKET_TCP;
 
   if (host == NULL)
+  {
     con->socket.tcp.host= NULL;
+  }
   else
   {
     con->socket.tcp.host= con->socket.tcp.host_buffer;
@@ -247,19 +255,35 @@
 
 const char *drizzle_con_user(const drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return NULL;
+  }
+
   return con->user;
 }
 
 const char *drizzle_con_password(const drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return NULL;
+  }
+
   return con->password;
 }
 
-void drizzle_con_set_auth(drizzle_con_st *con, const char *user,
-                          const char *password)
+void drizzle_con_set_auth(drizzle_con_st *con, const char *user, const char *password)
 {
+  if (con == NULL)
+  {
+    return;
+  }
+
   if (user == NULL)
+  {
     con->user[0]= 0;
+  }
   else
   {
     strncpy(con->user, user, DRIZZLE_MAX_USER_SIZE);
@@ -368,7 +392,7 @@
 
 uint32_t drizzle_con_server_version_number(const drizzle_con_st *con)
 {
-  if (con == NULL)
+  if (con == NULL or con->server_version)
   {
     return 0;
   }
@@ -1044,6 +1068,10 @@
 
 drizzle_return_t drizzle_state_connect(drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
   drizzle_log_debug(con->drizzle, "drizzle_state_connect");
 
   if (con->fd != -1)
@@ -1152,6 +1180,10 @@
 
 drizzle_return_t drizzle_state_connecting(drizzle_con_st *con)
 {
+  if (con == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
   drizzle_log_debug(con->drizzle, "drizzle_state_connecting");
 
   while (1)
@@ -1189,7 +1221,10 @@
 
 drizzle_return_t drizzle_state_read(drizzle_con_st *con)
 {
-  drizzle_return_t ret;
+  if (con == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
 
   drizzle_log_debug(con->drizzle, "drizzle_state_read");
 
@@ -1207,9 +1242,11 @@
     /* non-blocking mode: return IO_WAIT instead of attempting to read. This
      * avoids reading immediately after writing a command, which typically
      * returns EAGAIN. This improves performance. */
-    ret= drizzle_con_set_events(con, POLLIN);
+    drizzle_return_t ret= drizzle_con_set_events(con, POLLIN);
     if (ret != DRIZZLE_RETURN_OK)
+    {
       return ret;
+    }
     return DRIZZLE_RETURN_IO_WAIT;
   }
 
@@ -1271,9 +1308,11 @@
       {
         /* clear the read ready flag */
         con->revents&= ~POLLIN;
-        ret= drizzle_con_set_events(con, POLLIN);
+        drizzle_return_t ret= drizzle_con_set_events(con, POLLIN);
         if (ret != DRIZZLE_RETURN_OK)
+        {
           return ret;
+        }
 
         if (con->drizzle->options.is_non_blocking)
         {
@@ -1322,7 +1361,10 @@
 
 drizzle_return_t drizzle_state_write(drizzle_con_st *con)
 {
-  drizzle_return_t ret;
+  if (con == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
 
   drizzle_log_debug(con->drizzle, "drizzle_state_write");
 
@@ -1380,7 +1422,7 @@
     {
       if (errno == EAGAIN)
       {
-        ret= drizzle_con_set_events(con, POLLOUT);
+        drizzle_return_t ret= drizzle_con_set_events(con, POLLOUT);
         if (ret != DRIZZLE_RETURN_OK)
         {
           return ret;
@@ -1436,6 +1478,12 @@
   int opt;
   drizzle_con_st *new_con;
 
+  if (con == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
+
+
   for (; con->addrinfo_next != NULL;
        con->addrinfo_next= con->addrinfo_next->ai_next)
   {
@@ -1508,7 +1556,7 @@
     }
     else
     {
-      new_con= drizzle_con_clone(con->drizzle, NULL, con);
+      new_con= drizzle_con_clone(con->drizzle, con);
       if (new_con == NULL)
       {
         closesocket(fd);
@@ -1531,7 +1579,9 @@
 
   /* Report last socket() error if we couldn't find an address to bind. */
   if (con->fd == -1)
+  {
     return DRIZZLE_RETURN_ERRNO;
+  }
 
   drizzle_state_pop(con);
   return DRIZZLE_RETURN_OK;
@@ -1547,6 +1597,12 @@
   struct linger linger;
   struct timeval waittime;
 
+  if (con == NULL)
+  {
+    return DRIZZLE_RETURN_INVALID_ARGUMENT;
+  }
+
+
   ret= 1;
 
 #ifdef _WIN32

=== modified file 'libdrizzle-2.0/drizzle.cc'
--- libdrizzle-2.0/drizzle.cc	2011-11-24 03:44:17 +0000
+++ libdrizzle-2.0/drizzle.cc	2011-11-26 23:14:59 +0000
@@ -107,7 +107,6 @@
   {
     return NULL;
   }
-  drizzle->options.is_allocated= true;
 
   /* @todo remove this default free flag with new API. */
   drizzle->options.is_free_objects= true;
@@ -145,13 +144,9 @@
     return NULL;
   }
 
-  bool cache_state= drizzle->options.is_allocated;
-  drizzle->options= source->options;
-  drizzle->options.is_allocated= cache_state;
-
   for (drizzle_con_st* con= source->con_list; con != NULL; con= con->next)
   {
-    if (drizzle_con_clone(drizzle, NULL, con) == NULL)
+    if (drizzle_con_clone(drizzle, con) == NULL)
     {
       drizzle_free(drizzle);
       return NULL;
@@ -181,10 +176,7 @@
 
   free(drizzle->pfds);
 
-  if (drizzle->options.is_allocated)
-  {
-    delete drizzle;
-  }
+  delete drizzle;
 #if defined(_WIN32)
   /* if it is MS windows, invoke WSACleanup() at the end*/
   WSACleanup();
@@ -283,20 +275,24 @@
   drizzle->event_watch_context= context;
 }
 
-drizzle_con_st *drizzle_con_create(drizzle_st *drizzle, drizzle_con_st *con)
+drizzle_con_st *drizzle_con_create(drizzle_st *drizzle)
 {
+  if (drizzle == NULL)
+  {
+    return NULL;
+  }
+
+  drizzle_con_st *con= new (std::nothrow) drizzle_con_st;
+
   if (con == NULL)
   {
-    con= new (std::nothrow) drizzle_con_st;
-    con->_options.is_allocated= true;
-  }
-  else
-  {
-    con->options= 0;
+    return NULL;
   }
 
   if (drizzle->con_list != NULL)
+  {
     drizzle->con_list->prev= con;
+  }
 
   con->next= drizzle->con_list;
   con->prev= NULL;
@@ -351,32 +347,31 @@
   return con;
 }
 
-drizzle_con_st *drizzle_con_clone(drizzle_st *drizzle, drizzle_con_st *con,
-                                  const drizzle_con_st *from)
+drizzle_con_st *drizzle_con_clone(drizzle_st *drizzle, drizzle_con_st *source)
 {
-  con= drizzle_con_create(drizzle, con);
+  drizzle_con_st *con= drizzle_con_create(drizzle);
   if (con == NULL)
   {
     return NULL;
   }
 
   /* Clear "operational" options such as IO status. */
-  con->options|= (from->options & ~(DRIZZLE_CON_ALLOCATED|DRIZZLE_CON_READY|
+  con->options|= (source->options & ~(DRIZZLE_CON_ALLOCATED|DRIZZLE_CON_READY|
                   DRIZZLE_CON_NO_RESULT_READ|DRIZZLE_CON_IO_READY|
                   DRIZZLE_CON_LISTEN));
-  con->backlog= from->backlog;
-  strcpy(con->schema, from->schema);
-  strcpy(con->password, from->password);
-  strcpy(con->user, from->user);
+  con->backlog= source->backlog;
+  strcpy(con->schema, source->schema);
+  strcpy(con->password, source->password);
+  strcpy(con->user, source->user);
 
-  switch (from->socket_type)
+  switch (source->socket_type)
   {
   case DRIZZLE_CON_SOCKET_TCP:
-    drizzle_con_set_tcp(con, from->socket.tcp.host, from->socket.tcp.port);
+    drizzle_con_set_tcp(con, source->socket.tcp.host, source->socket.tcp.port);
     break;
 
   case DRIZZLE_CON_SOCKET_UDS:
-    drizzle_con_set_uds(con, from->socket.uds.sockaddr.sun_path);
+    drizzle_con_set_uds(con, source->socket.uds.sockaddr.sun_path);
     break;
   }
 
@@ -420,10 +415,7 @@
   }
   con->drizzle->con_count--;
 
-  if (con->_options.is_allocated)
-  {
-    delete con;
-  }
+  delete con;
 }
 
 void drizzle_con_free_all(drizzle_st *drizzle)
@@ -557,13 +549,13 @@
  * Client Definitions
  */
 
-drizzle_con_st *drizzle_con_add_tcp(drizzle_st *drizzle, drizzle_con_st *con,
+drizzle_con_st *drizzle_con_add_tcp(drizzle_st *drizzle,
                                     const char *host, in_port_t port,
                                     const char *user, const char *password,
                                     const char *db,
                                     drizzle_con_options_t options)
 {
-  con= drizzle_con_create(drizzle, con);
+  drizzle_con_st *con= drizzle_con_create(drizzle);
   if (con == NULL)
   {
     return NULL;
@@ -577,12 +569,12 @@
   return con;
 }
 
-drizzle_con_st *drizzle_con_add_uds(drizzle_st *drizzle, drizzle_con_st *con,
+drizzle_con_st *drizzle_con_add_uds(drizzle_st *drizzle,
                                     const char *uds, const char *user,
                                     const char *password, const char *db,
                                     drizzle_con_options_t options)
 {
-  con= drizzle_con_create(drizzle, con);
+  drizzle_con_st *con= drizzle_con_create(drizzle);
   if (con == NULL)
   {
     return NULL;
@@ -601,12 +593,11 @@
  */
 
 drizzle_con_st *drizzle_con_add_tcp_listen(drizzle_st *drizzle,
-                                           drizzle_con_st *con,
                                            const char *host, in_port_t port,
                                            int backlog,
                                            drizzle_con_options_t options)
 {
-  con= drizzle_con_create(drizzle, con);
+  drizzle_con_st *con= drizzle_con_create(drizzle);
   if (con == NULL)
   {
     return NULL;
@@ -620,11 +611,10 @@
 }
 
 drizzle_con_st *drizzle_con_add_uds_listen(drizzle_st *drizzle,
-                                           drizzle_con_st *con,
                                            const char *uds, int backlog,
                                            drizzle_con_options_t options)
 {
-  con= drizzle_con_create(drizzle, con);
+  drizzle_con_st *con= drizzle_con_create(drizzle);
   if (con == NULL)
   {
     return NULL;
@@ -637,16 +627,22 @@
   return con;
 }
 
-drizzle_con_st *drizzle_con_accept(drizzle_st *drizzle, drizzle_con_st *con,
+drizzle_con_st *drizzle_con_accept(drizzle_st *drizzle,
                                    drizzle_return_t *ret_ptr)
 {
+  drizzle_return_t unused;
+  if (ret_ptr == NULL)
+  {
+    ret_ptr= &unused;
+  }
+
   while (1)
   {
     if (drizzle_con_st* ready= drizzle_con_ready_listen(drizzle))
     {
       int fd= accept(ready->fd, NULL, NULL);
 
-      con= drizzle_con_create(drizzle, con);
+      drizzle_con_st *con= drizzle_con_create(drizzle);
       if (con == NULL)
       {
         (void)closesocket(fd);
@@ -677,12 +673,16 @@
     for (drizzle_con_st* ready= drizzle->con_list; ready != NULL; ready= ready->next)
     {
       if (ready->options & DRIZZLE_CON_LISTEN)
+      {
         drizzle_con_set_events(ready, POLLIN);
+      }
     }
 
     *ret_ptr= drizzle_con_wait(drizzle);
     if (*ret_ptr != DRIZZLE_RETURN_OK)
+    {
       return NULL;
+    }
   }
 }
 
@@ -693,6 +693,10 @@
 void drizzle_set_error(drizzle_st *drizzle, const char *function,
                        const char *format, ...)
 {
+  if (drizzle == NULL)
+  {
+    return;
+  }
   char log_buffer[DRIZZLE_MAX_ERROR_SIZE];
 
   size_t size= strlen(function);
@@ -737,6 +741,11 @@
 {
   char log_buffer[DRIZZLE_MAX_ERROR_SIZE];
 
+  if (drizzle == NULL)
+  {
+    return;
+  }
+
   if (drizzle->log_fn == NULL)
   {
     printf("%5s: ", drizzle_verbose_name(verbose));

=== modified file 'libdrizzle-2.0/drizzle.h'
--- libdrizzle-2.0/drizzle.h	2011-11-24 03:44:17 +0000
+++ libdrizzle-2.0/drizzle.h	2011-11-26 23:14:59 +0000
@@ -335,7 +335,7 @@
  *  failure this will be NULL.
  */
 DRIZZLE_API
-drizzle_con_st *drizzle_con_create(drizzle_st *drizzle, drizzle_con_st *con);
+drizzle_con_st *drizzle_con_create(drizzle_st *drizzle);
 
 /**
  * Clone a connection structure.
@@ -347,8 +347,7 @@
  * @return Same return as drizzle_con_create().
  */
 DRIZZLE_API
-drizzle_con_st *drizzle_con_clone(drizzle_st *drizzle, drizzle_con_st *con,
-                                  const drizzle_con_st *from);
+drizzle_con_st *drizzle_con_clone(drizzle_st *drizzle, drizzle_con_st *con);
 
 DRIZZLE_API
 drizzle_return_t drizzle_set_option(drizzle_st *drizzle, drizzle_options_t arg, bool set);

=== modified file 'libdrizzle-2.0/drizzle_client.h'
--- libdrizzle-2.0/drizzle_client.h	2011-11-07 15:04:16 +0000
+++ libdrizzle-2.0/drizzle_client.h	2011-11-26 23:14:59 +0000
@@ -81,7 +81,7 @@
  * @return Same return as drizzle_con_create().
  */
 DRIZZLE_API
-drizzle_con_st *drizzle_con_add_tcp(drizzle_st *drizzle, drizzle_con_st *con,
+drizzle_con_st *drizzle_con_add_tcp(drizzle_st *drizzle,
                                     const char *host, in_port_t port,
                                     const char *user, const char *password,
                                     const char *db,
@@ -101,7 +101,7 @@
  * @return Same return as drizzle_con_create().
  */
 DRIZZLE_API
-drizzle_con_st *drizzle_con_add_uds(drizzle_st *drizzle, drizzle_con_st *con,
+drizzle_con_st *drizzle_con_add_uds(drizzle_st *drizzle,
                                     const char *uds, const char *user,
                                     const char *password, const char *db,
                                     drizzle_con_options_t options);

=== modified file 'libdrizzle-2.0/drizzle_server.h'
--- libdrizzle-2.0/drizzle_server.h	2011-11-07 15:04:16 +0000
+++ libdrizzle-2.0/drizzle_server.h	2011-11-26 23:14:59 +0000
@@ -79,7 +79,6 @@
  */
 DRIZZLE_API
 drizzle_con_st *drizzle_con_add_tcp_listen(drizzle_st *drizzle,
-                                           drizzle_con_st *con,
                                            const char *host, in_port_t port,
                                            int backlog,
                                            drizzle_con_options_t options);
@@ -89,7 +88,6 @@
  *
  * @param[in] drizzle Drizzle structure previously initialized with
  *  drizzle_create() or drizzle_clone().
- * @param[in] con Caller allocated structure, or NULL to allocate one.
  * @param[in] uds Path to unix domain socket to use for listening.
  * @param[in] backlog Number of backlog connections passed to listen().
  * @param[in] options Drizzle connection options to add.
@@ -97,7 +95,6 @@
  */
 DRIZZLE_API
 drizzle_con_st *drizzle_con_add_uds_listen(drizzle_st *drizzle,
-                                           drizzle_con_st *con,
                                            const char *uds, int backlog,
                                            drizzle_con_options_t options);
 
@@ -116,12 +113,11 @@
  *
  * @param[in] drizzle Drizzle structure previously initialized with
  *  drizzle_create() or drizzle_clone().
- * @param[in] con Caller allocated structure, or NULL to allocate one.
  * @param[out] ret_ptr Standard drizzle return value.
  * @return Same return as drizzle_con_create().
  */
 DRIZZLE_API
-drizzle_con_st *drizzle_con_accept(drizzle_st *drizzle, drizzle_con_st *con,
+drizzle_con_st *drizzle_con_accept(drizzle_st *drizzle,
                                    drizzle_return_t *ret_ptr);
 
 /** @} */

=== modified file 'libdrizzle-2.0/include.am'
--- libdrizzle-2.0/include.am	2011-11-24 00:05:44 +0000
+++ libdrizzle-2.0/include.am	2011-11-26 23:14:59 +0000
@@ -130,8 +130,6 @@
 			 libdrizzle-2.0/field_server.h \
 			 libdrizzle-2.0/handshake_client.h \
 			 libdrizzle-2.0/handshake_server.h \
-			 libdrizzle-2.0/libdrizzle.h \
-			 libdrizzle-2.0/libdrizzle.hpp \
 			 libdrizzle-2.0/limits.h \
 			 libdrizzle-2.0/query.h \
 			 libdrizzle-2.0/result.h \
@@ -145,9 +143,11 @@
 
 noinst_HEADERS+= \
 		 libdrizzle-2.0/common.h \
+		 libdrizzle-2.0/conn_local.h \
 		 libdrizzle-2.0/drizzle_local.h \
-		 libdrizzle-2.0/conn_local.h \
+		 libdrizzle-2.0/libdrizzle.h \
+		 libdrizzle-2.0/libdrizzle.hpp \
 		 libdrizzle-2.0/pack.h \
+		 libdrizzle-2.0/sha1.h \
 		 libdrizzle-2.0/state.h \
-		 libdrizzle-2.0/structs.h \
-		 libdrizzle-2.0/sha1.h
+		 libdrizzle-2.0/structs.h

=== modified file 'libdrizzle-2.0/libdrizzle.hpp'
--- libdrizzle-2.0/libdrizzle.hpp	2011-11-24 00:05:44 +0000
+++ libdrizzle-2.0/libdrizzle.hpp	2011-11-26 23:14:59 +0000
@@ -148,46 +148,56 @@
 public:
   explicit connection_c(drizzle_c& drizzle)
   {
-    drizzle_con_create(drizzle, *this);
+    b_= drizzle_con_create(drizzle);
+
+    if (b_ == NULL)
+    {
+      throw "drizzle_con_create() failed";
+    }
     read_conf_files();
   }
 
   ~connection_c()
   {
-    drizzle_con_free(*this);
+    drizzle_con_free(b_);
   }
 
   operator drizzle_con_st*()
   {
-    return &b_;
+    return b_;
   }
 
   const char* error()
   {
-    return drizzle_con_error(*this);
+    return drizzle_con_error(b_);
   }
 
   void set_tcp(const char* host, in_port_t port)
   {
-    drizzle_con_set_tcp(*this, host, port);
+    drizzle_con_set_tcp(b_, host, port);
   }
 
   void set_auth(const char* user, const char* password)
   {
-    drizzle_con_set_auth(*this, user, password);
+    drizzle_con_set_auth(b_, user, password);
   }
 
   void set_db(const char* db)
   {
-    drizzle_con_set_db(*this, db);
+    drizzle_con_set_db(b_, db);
   }
 
   drizzle_return_t query(result_c& result, const char* str, size_t str_size)
   {
     drizzle_return_t ret;
+
     drizzle_query(*this, result, str, str_size, &ret);
+
     if (!ret)
+    {
       ret = drizzle_result_buffer(result);
+    }
+
     return ret;
   }
 
@@ -205,7 +215,10 @@
   {
     result_c result;
     if (query(result, str, str_size))
+    {
       throw bad_query(error());
+    }
+
     return result;
   }
 
@@ -221,7 +234,7 @@
 private:
   void read_conf_files();
 
-  drizzle_con_st b_;
+  drizzle_con_st *b_;
 };
 
 class query_c

=== modified file 'plugin/slave/queue_producer.cc'
--- plugin/slave/queue_producer.cc	2011-11-24 00:05:44 +0000
+++ plugin/slave/queue_producer.cc	2011-11-26 23:14:59 +0000
@@ -18,6 +18,8 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include <config.h>
+
 #include <plugin/slave/queue_producer.h>
 #include <drizzled/errmsg_print.h>
 #include <drizzled/sql/result_set.h>
@@ -142,7 +144,7 @@
     return false;
   }
   
-  if (drizzle_con_create(_drizzle, &_connection) == NULL)
+  if ((_connection= drizzle_con_create(_drizzle)) == NULL)
   {
     _last_return= DRIZZLE_RETURN_INTERNAL_ERROR;
     _last_error_message= "Replication slave: ";
@@ -151,10 +153,10 @@
     return false;
   }
   
-  drizzle_con_set_tcp(&_connection, _master_host.c_str(), _master_port);
-  drizzle_con_set_auth(&_connection, _master_user.c_str(), _master_pass.c_str());
+  drizzle_con_set_tcp(_connection, _master_host.c_str(), _master_port);
+  drizzle_con_set_auth(_connection, _master_user.c_str(), _master_pass.c_str());
 
-  drizzle_return_t ret= drizzle_con_connect(&_connection);
+  drizzle_return_t ret= drizzle_con_connect(_connection);
 
   if (ret != DRIZZLE_RETURN_OK)
   {
@@ -177,7 +179,7 @@
 
   _is_connected= false;
 
-  if (drizzle_quit(&_connection, &result, &ret) == NULL)
+  if (drizzle_quit(_connection, &result, &ret) == NULL)
   {
     _last_return= ret;
     drizzle_result_free(&result);
@@ -246,7 +248,7 @@
 
   drizzle_return_t ret;
   drizzle_result_st result;
-  drizzle_query_str(&_connection, &result, sql.c_str(), &ret);
+  drizzle_query_str(_connection, &result, sql.c_str(), &ret);
   
   if (ret != DRIZZLE_RETURN_OK)
   {
@@ -418,7 +420,7 @@
 
   drizzle_return_t ret;
   drizzle_result_st result;
-  drizzle_query_str(&_connection, &result, sql.c_str(), &ret);
+  drizzle_query_str(_connection, &result, sql.c_str(), &ret);
   
   if (ret != DRIZZLE_RETURN_OK)
   {

=== modified file 'plugin/slave/queue_producer.h'
--- plugin/slave/queue_producer.h	2011-11-24 00:05:44 +0000
+++ plugin/slave/queue_producer.h	2011-11-26 23:14:59 +0000
@@ -125,7 +125,7 @@
   uint32_t _master_id;
 
   drizzle_st *_drizzle;
-  drizzle_con_st _connection;
+  drizzle_con_st *_connection;
   drizzle_return_t _last_return;
 
   bool _is_connected;

=== modified file 'plugin/transaction_log/utilities/transaction_log_connection.cc'
--- plugin/transaction_log/utilities/transaction_log_connection.cc	2011-11-24 00:05:44 +0000
+++ plugin/transaction_log/utilities/transaction_log_connection.cc	2011-11-26 23:14:59 +0000
@@ -25,10 +25,9 @@
 
 TransactionLogConnection::TransactionLogConnection(string &host, uint16_t port,
                                                    string &username, string &password,
-                                                   bool drizzle_protocol)
-  :
-    hostName(host),
-    drizzleProtocol(drizzle_protocol)
+                                                   bool drizzle_protocol) :
+  hostName(host),
+  drizzleProtocol(drizzle_protocol)
 {
   if (host.empty())
   {
@@ -40,16 +39,22 @@
   if (drizzle == NULL)
   {
     errorHandler(NULL, DRIZZLE_RETURN_MEMORY, "drizzle_create() failed");
-    throw 1;
-  }
-  drizzle_con_create(drizzle, &connection);
-  drizzle_con_set_tcp(&connection, (char *)host.c_str(), port);
-  drizzle_con_set_auth(&connection, (char *)username.c_str(), (char *)password.c_str());
-
-  drizzle_con_add_options(&connection,
-    drizzle_protocol ? DRIZZLE_CON_EXPERIMENTAL : DRIZZLE_CON_MYSQL);
-
-  drizzle_return_t ret= drizzle_con_connect(&connection);
+    throw "drizzle_create() failed";
+  }
+
+  connection= drizzle_con_create(drizzle);
+  if (connection == NULL)
+  {
+    errorHandler(NULL, DRIZZLE_RETURN_MEMORY, "drizzle_create() failed");
+    throw "drizzle_con_create() failed";
+  }
+  drizzle_con_set_tcp(connection, (char *)host.c_str(), port);
+  drizzle_con_set_auth(connection, (char *)username.c_str(), (char *)password.c_str());
+
+  drizzle_con_add_options(connection,
+                          drizzle_protocol ? DRIZZLE_CON_EXPERIMENTAL : DRIZZLE_CON_MYSQL);
+
+  drizzle_return_t ret= drizzle_con_connect(connection);
 
   if (ret != DRIZZLE_RETURN_OK)
   {
@@ -63,7 +68,7 @@
 {
   drizzle_return_t ret;
 
-  if (drizzle_query_str(&connection, result, str_query.c_str(), &ret) == NULL ||
+  if (drizzle_query_str(connection, result, str_query.c_str(), &ret) == NULL ||
       ret != DRIZZLE_RETURN_OK)
   {
     if (ret == DRIZZLE_RETURN_ERROR_CODE)
@@ -75,7 +80,7 @@
     else
     {
       cerr << "Error executing query: " <<
-        drizzle_con_error(&connection) << endl;
+        drizzle_con_error(connection) << endl;
       drizzle_result_free(result);
     }
     return;
@@ -84,18 +89,18 @@
   if (drizzle_result_buffer(result) != DRIZZLE_RETURN_OK)
   {
     cerr << "Could not buffer result: " <<
-        drizzle_con_error(&connection) << endl;
+        drizzle_con_error(connection) << endl;
     drizzle_result_free(result);
     return;
   }
 }
 
 void TransactionLogConnection::errorHandler(drizzle_result_st *res,
-  drizzle_return_t ret, const char *when)
+                                            drizzle_return_t ret, const char *when)
 {
   if (res == NULL)
   {
-    cerr << "Got error: " << drizzle_con_error(&connection) << " " << when << endl;
+    cerr << "Got error: " << drizzle_con_error(connection) << " " << when << endl;
   }
   else if (ret == DRIZZLE_RETURN_ERROR_CODE)
   {

=== modified file 'plugin/transaction_log/utilities/transaction_log_connection.h'
--- plugin/transaction_log/utilities/transaction_log_connection.h	2011-11-24 00:05:44 +0000
+++ plugin/transaction_log/utilities/transaction_log_connection.h	2011-11-26 23:14:59 +0000
@@ -39,7 +39,7 @@
 
 private:
   drizzle_st *drizzle;
-  drizzle_con_st connection;
+  drizzle_con_st *connection;
   std::string hostName;
   bool drizzleProtocol;
 };

